/*
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2020, 2021. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *
 */
import { IBMZappDocument } from './zapp-schema';
/**
 * A meta-data representation of a zapp file
 * providing path and type.
 */
export interface ZappFile {
    path: string;
    type: 'json' | 'yaml';
}
/**
 * Represent ZAPP documents that are stored either as yaml or json files.
 * Features various factory method to create instances for workspace locations or absolute file names.
 */
export declare class ZappDocumentInstance {
    private zappYawn;
    /** If true the document was parsed and validated successfully.
     *  If false then pasing or validation failed. Check the errorMessages members then.  */
    isValid: boolean;
    /** Provides meta-data for the file used to create the instance using the ZappFile interface */
    file: ZappFile | undefined;
    /** Error message resulting from IO errors, parsing, or validations. TODO: requires localization */
    errorMessages: string | undefined;
    private static validationFunction;
    private static initAjvValidator;
    /**
     * Creates an instance of ZappDocumentInstance. Protected, use factory methods instead.
     * @param {string} zappYamlString String representation of the yaml used to create the instance
     * @param {string} [errorMessages] Optionaly provide error messages to be preserved such as IO errors from reading a file
     */
    protected constructor(zappYamlString: string, errorMessages?: string);
    /**
     * Creates a ZappDocumentInstance with a yaml string.
     * @param {string} zappYamlString
     * @returns {ZappDocumentInstance}
     */
    static createFromYamlString(zappYamlString: string): ZappDocumentInstance;
    /**
     * Creates a ZappDocumentInstance with a json string.
     * @param {string} zappJsonString
     * @returns {ZappDocumentInstance}
     */
    static createFromJsonString(zappJsonString: string): ZappDocumentInstance;
    /**
     * Creates a ZappDocumentInstance with a yaml file path
     * @param {string} filepath absolute path name
     * @returns {Promise<ZappDocumentInstance>} promise that will resolve into an instantiated ZappDocumentInstance.
     *          If no document was found or it contained errors then these errors will be populated in the object.
     */
    static createFromYamlFile(filepath: string): Promise<ZappDocumentInstance>;
    /**
     * Creates a ZappDocumentInstance with a json file path
     * @param {string} filepath absolute path name
     * @returns {Promise<ZappDocumentInstance>} promise that will resolve into an instantiated ZappDocumentInstance.
     *          If no document was found or it contained errors then these errors will be populated in the object.
     */
    static createFromJsonFile(filepath: string): Promise<ZappDocumentInstance>;
    /**
     * When running ZAPP in an editor then the editor will write file updates in a parallel process
     * than the ZAPP code that runs in an extension host in a parallel node instance. We observed
     * that sometime node's fs.readFile will then return an empty string. The theory is that the file is
     * still locked by the write process because the file contents have not yet been fully flushed to
     * the disk by the other process. As a work around we want to retry reading a file when it returns
     * empty. When an exception is throw then it will not retry.
     *
     * TODO: all the watcher and IO code should be moved into its own replaceable modules.
     *
     * @param filepath Absolute path to the file to read
     * @returns the contents of the file read or throws an exception if multiple attempts failed
     */
    private static readFileWithRetry;
    /**
     * Creates a ZappDocumentInstance from a file. Checks the file type automatically.
     * @param {string} filepath absolute path name
     * @returns {Promise<ZappDocumentInstance>} promise that will resolve into an instantiated ZappDocumentInstance.
     *          If no document was found or it contained errors then these errors will be populated in the object.
     */
    static createFromFile(filepath: string): Promise<ZappDocumentInstance>;
    /**
     * Creates a ZappDocumentInstance by looking for a zapp file in a directory.
     * Checks the file type automatically. Will always prioritize yaml over json.
     * @param {string} workspacePath absolute path name
     * @returns {Promise<ZappDocumentInstance>} promise that will resolve into an instantiated ZappDocumentInstance.
     *          If no document was found or it contained errors then these errors will be populated in the object.
     */
    static loadFromWorkspace(workspacePath: string): Promise<ZappDocumentInstance>;
    /**
     * Looks for zapp files in a directory. Will always prioritize yaml over json.
     * @param {string} workspacePath absolute path name
     * @returns {(ZappFile | undefined)} An instance of ZappFile or undefined if not found
     */
    static findZappFile(workspacePath: string): ZappFile | undefined;
    /**
     * @param {string} filename string that represent a filename; not a path
     * @returns {boolean} true if filename is either zapp.json, zapp.yml, or zapp.yaml
     */
    static isZappFileName(filename: string): boolean;
    /**
     * @returns {(IBMZappDocument | undefined)} returns the ZAPP document as an instance of the IBMZappDocument interface
     */
    getAsDocument(): IBMZappDocument | undefined;
    /**
     * @returns {(string | undefined)} returns the ZAPP document as YAML string
     */
    getAsYaml(): string | undefined;
    /**
     * @returns {(Record<string, unknown> | undefined)} returns the ZAPP document as untyped object
     */
    protected getAsObject(): Record<string, unknown> | undefined;
    /**
     * If the document instance was read from a file and/or the file is populated with path and type
     * then write it back to the file system replacing the previous file.
     * TODO: expose via the document cache to deal with watchers correctly.
     */
    write(): void;
    /**
     * Can be called to check that the document complies with the ZAPP json-schema.
     * If validation errors are found then they will added to the errorMessages member.
     * @returns {boolean} false if validation errors were found
     */
    validate(): boolean;
    private appendErrorMessage;
    private prependErrorMessage;
}
