/*
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2020, 2021. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *
 */
import { Session, ICommandHandler, IHandlerParameters, IProfile, ProfileInfo } from '@zowe/imperative';
import { IRestApiResponse } from '../api/doc/IRestApiResponse';
/**
 * This class is used by the various rest handlers as the base class for their implementation.
 * All handlers should extend this class whenever possible
 */
export declare abstract class RestBaseHandler implements ICommandHandler {
    /**
     * Local reference to commandParameters for convenience
     */
    protected params: IHandlerParameters;
    /**
     * This will grab the rest profile and create a session before calling the subclass
     * {@link RestBaseHandler#processWithSession} method.
     *
     * @param {IHandlerParameters} commandParameters Command parameters sent by imperative.
     *
     * @returns {Promise<void>}
     */
    process(commandParameters: IHandlerParameters): Promise<void>;
    /**
     * This is called by the {@link RestBaseHandler#usingTeamConfig} and {@link RestBaseHandler#usingYamlProfile}
     * after it creates a session. Should be used so that every class does not have to instantiate the session object.
     *
     * @param {IHandlerParameters} commandParameters Command parameters sent to the handler.
     * @param {Session} session The session object generated from the rest profile.
     * @param {IProfile} restProfile The rest profile that was loaded for the command.
     *
     * @returns {Promise<IRestApiResponse>} The response from the underlying rest api call.
     */
    abstract processWithSession(commandParameters: IHandlerParameters, session: Session, restProfile: IProfile): Promise<IRestApiResponse>;
    /**
     * This is called by the {@link RestBaseHandler#process} to create a session from config file.
     * Should be used so that every class does not have to instantiate the session object.
     *
     * @param {IHandlerParameters} parms Command parameters sent to the handler.
     * @param {Session} session The session object generated from the rest profile.
     * @param {ProfileInfo} profileInfo The rest profile that was loaded for the command.
     *
     * @returns {Promise<void>} The response from the underlying rest api call.
     */
    usingTeamConfig(profileInfo: ProfileInfo): Promise<void>;
    /**
     * This is called by the {@link RestBaseHandler#process} to create a session from yaml profiles.
     * Should be used so that every class does not have to instantiate the session object.
     *
     * @param {IHandlerParameters} parms Command parameters sent to the handler.
     * @param {Session} session The session object generated from the rest profile.
     *
     * @returns {Promise<void>} The response from the underlying rest api call.
     */
    usingYamlProfiles(profile: IProfile): Promise<void>;
    /**
     * Check return codes for messages in case we don't receive a bad HTTP status code but
     * do receive a bad RC in one of the messages returned by the API
     */
    protected checkReturnCode(response: IRestApiResponse): void;
}
